from liquid.filter import Filter
from liquid.filter import string_required
from liquid.filter import no_args


class StylesheetTag(Filter):
    """Wrap a URL in an HTML stylesheet tag."""

    name = "stylesheet_tag"

    @string_required
    @no_args
    def __call__(self, url):
        return f'<link href="{url}" rel="stylesheet" type="text/css" media="all" />'


class ScriptTag(Filter):
    """Wrap a URL in an HTML script tag."""

    name = "script_tag"

    @string_required
    @no_args
    def __call__(self, url):
        return f'<script src="{url}" type="text/javascript"></script>'



import json

from liquid.context import get_item
from liquid.filter import Filter
from liquid.filter import no_args


class JSON(Filter):
    """Serialize objects as a JSON (JavaScript Object Notation) formatted string.

    Args:
        default: A 'default' function passed to json.dumps. This function is called
            in the event that the JSONEncoder does not know how to serialize an object.
    """

    name = "json"

    def __init__(self, env, default=None):
        super().__init__(env)
        self.default = default

    @no_args
    def __call__(self, obj):
        return json.dumps(obj, default=self.default)


class Translate(Filter):
    """Replace translation keys with strings for the current locale.

    Tries to read the locale from the current template context, falling back to
    "default" if the key "locale" does not exist.

    Args:
        locales: A mapping of locale name to translation key mapping. If locales
            is `None`, the default, the translation key will be returned unchanged.
    """

    name = "t"
    with_context = True

    def __init__(self, env, locales=None):
        super().__init__(env)
        self.locales = locales or {}

    def __call__(self, key, *, context, **kwargs):
        locale = context.resolve("locale", default="default")
        translations = self.locales.get(locale, {})

        key = str(key)
        path = key.split(".")

        val = get_item(translations, *path, default=key)
        return self.env.from_string(val).render(**kwargs)



from liquid.filter import Filter


class Index(Filter):
    """Return the first zero-based index of an item in an array. Or None if
    the items is not in the array.
    """

    name = "index"

    def __call__(self, arr, obj):
        try:
            return arr.index(obj)
        except ValueError:
            return None



import setuptools


with open("README.rst", "r") as fd:
    long_description = fd.read()

setuptools.setup(
    name="python-liquid-extra",
    version="0.1.0",
    description="Extra tags an filters for python-liquid.",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/jg-rp/liquid-extra",
    packages=setuptools.find_packages(exclude=["tests*"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=["python-liquid>=0.6.2"],
    test_suite="tests",
    python_requires=">=3.7",
)



from __future__ import annotations

import sys

from functools import partial
from typing import TYPE_CHECKING

from liquid.lex import _tokenize
from liquid.lex import _compile_rules
from liquid.lex import STRING_PATTERN
from liquid.lex import boolean_expression_keywords

from liquid.parse import expect
from liquid.parse import ExpressionParser
from liquid.parse import Precedence

from liquid.context import Context
from liquid.exceptions import LiquidTypeError

from liquid.expression import PrefixExpression
from liquid.expression import is_truthy

from liquid.stream import TokenStream

from liquid.token import TOKEN_EXPRESSION
from liquid.token import TOKEN_LPAREN
from liquid.token import TOKEN_RPAREN
from liquid.token import TOKEN_EOF
from liquid.token import TOKEN_FLOAT
from liquid.token import TOKEN_INTEGER
from liquid.token import TOKEN_NEGATIVE
from liquid.token import TOKEN_STRING
from liquid.token import TOKEN_IDENTIFIER
from liquid.token import TOKEN_DOT
from liquid.token import TOKEN_LBRACKET
from liquid.token import TOKEN_RBRACKET
from liquid.token import TOKEN_ILLEGAL

from liquid.builtin.tags.if_tag import IfTag

if TYPE_CHECKING:  # pragma: no cover
    from liquid import Environment
    from liquid.expression import Expression


TOKEN_NOT = sys.intern("not")

boolean_not_expression_rules = (
    (TOKEN_FLOAT, r"\d+\.\d*"),
    (TOKEN_INTEGER, r"\d+"),
    (TOKEN_NEGATIVE, r"-"),
    (TOKEN_STRING, STRING_PATTERN),
    (TOKEN_IDENTIFIER, r"\w[a-zA-Z0-9_\-?]*"),
    (TOKEN_DOT, r"\."),
    (TOKEN_LBRACKET, r"\["),
    (TOKEN_RBRACKET, r"]"),
    (TOKEN_LPAREN, r"\("),
    (TOKEN_RPAREN, r"\)"),
    ("NEWLINE", r"\n"),
    ("OP", r"[!=<>]{1,2}"),
    ("SKIP", r"[ \t]+"),
    (TOKEN_ILLEGAL, r"."),
)


boolean_not_expression_keywords = frozenset([*boolean_expression_keywords, TOKEN_NOT])

tokenize_boolean_not_expression = partial(
    _tokenize,
    rules=_compile_rules(boolean_not_expression_rules),
    keywords=boolean_not_expression_keywords,
)


class NotPrefixExpression(PrefixExpression):
    def evaluate(self, context: Context):
        right = self.right.evaluate(context)

        if self.operator == "-":
            if isinstance(right, (int, float)):
                return -right
            raise LiquidTypeError(f"unknown operator {self.operator}{self.right}")

        if self.operator == TOKEN_NOT:
            return not is_truthy(right)

        raise LiquidTypeError(f"unknown operator {self.operator}")


class NotExpressionParser(ExpressionParser):
    def __init__(self):
        super().__init__()
        self.prefix_funcs[TOKEN_NOT] = self.parse_prefix_expression

        # For grouped expressions
        self.prefix_funcs[TOKEN_LPAREN] = self.parse_grouped_expression
        self.precedences[TOKEN_RPAREN] = Precedence.LOWEST

    def parse_prefix_expression(self, stream: TokenStream) -> NotPrefixExpression:
        tok = stream.current
        stream.next_token()

        exp = NotPrefixExpression(
            tok.value,
            right=self.parse_expression(stream, precedence=Precedence.PREFIX),
        )

        return exp

    def parse_grouped_expression(self, stream: TokenStream) -> Expression:
        stream.next_token()
        exp = self.parse_expression(stream)

        stream.next_token()
        while stream.current.type == TOKEN_RPAREN:
            stream.next_token()

        if stream.current.type != TOKEN_EOF:
            exp = self.parse_infix_expression(stream, left=exp)

        return exp


class IfNotTag(IfTag):
    """A drop-in replacement for the standard `if` tag that allows logical
    `not` operators in expressions."""

    def __init__(self, env: Environment):
        super().__init__(env)
        self.expression_parser = NotExpressionParser()

    def parse_expression(self, stream: TokenStream) -> Expression:
        expect(stream, TOKEN_EXPRESSION)
        expr_iter = tokenize_boolean_not_expression(stream.current.value)
        return self.expression_parser.parse_boolean_expression(TokenStream(expr_iter))
